<?php
/*
 * @package Joomla 1.5
 * @copyright Copyright (C) 2005 Open Source Matters. All rights reserved.
 * @license GNU/GPL v2 http://www.gnu.org/licenses/gpl-2.0.html, see LICENSE.php
 *
 * @module OpenSim Friends
 * @copyright Copyright (C) 2010 FoTo50 http://www.foto50.com/opensim/
 * @license GNU/GPL v2 http://www.gnu.org/licenses/gpl-2.0.html
 */
defined('_JEXEC') or die('Direct Access to this location is not allowed.');
 
class ModOpenSimFriendsHelper {
	public $friendlist = array();
	public $os_uid;			// if present, here is the OS UID
	public $os_settings;	// general Settings from the Component
	public $_osgrid_db;		// OpenSim Database Object
	public $currentuser;	// the current logged in user
	public $opensim;
	public $onlinecolor		= "#00ff00"; // Default online color
	public $offlinecolor	= "#ff0000"; // Default offline color
	public $namelength		= 0;

	public function __construct($namelength = 0) {
		$this->namelength = $namelength;
		$this->initOpenSim();						// generate the opensim object
		$this->getSettings();						// fetch the general Settings from the component at the beginning
		$this->_osgrid_db	= $this->getOSdb();		// load the external DB in an object
		$this->currentuser	=& JFactory::getUser();	// who is here now?
		$this->getUIDfromUser();					// and is he/she registered in OpenSim?
		$this->initFriendsList();					// Fill up the FriendList (if applicable)
	}

	public function _buildSettingsQuery() {
		$query = "SELECT #__opensim_config.* FROM #__opensim_config";
		return $query;
	}

	public function getSettings() {
		$db =& JFactory::getDBO();
		$query = $this->_buildSettingsQuery();
		$db->setQuery($query);
		$jOpenSimSettings = $db->loadAssocList();
		if(is_array($jOpenSimSettings) && count($jOpenSimSettings) > 0) {
			$settings = array();
			foreach($jOpenSimSettings AS $jOpenSimSetting) {
				if($jOpenSimSetting['configname'] == "lastnamelist") {
					$lastnames = explode("\n",$jOpenSimSetting['configvalue']);
					if(is_array($lastnames) && count($lastnames) > 0) {
						foreach($lastnames AS $lastname) {
							if(trim($lastname)) $settings['lastnames'][] = trim($lastname);
						}
					}
					$settings['lastnamelist'] = $jOpenSimSetting['configvalue'];
					continue;
				}
				switch($jOpenSimSetting['configtype']) {
					case "int":
						$settings[$jOpenSimSetting['configname']] = intval($jOpenSimSetting['configvalue']);
					break;
					case "float":
						$settings[$jOpenSimSetting['configname']] = floatval($jOpenSimSetting['configvalue']);
					break;
					default:
						$settings[$jOpenSimSetting['configname']] = strval($jOpenSimSetting['configvalue']);
					break;
				}
			}
		} else {
			return FALSE;
		}
		$this->os_settings					= $settings;
		$this->os_settings['lastDays']		= $settings['loginscreen_xdays'];
		$this->os_settings['gridstatus']	= $settings['loginscreen_gridstatus'];
		$this->os_settings['gridlines']		= $settings['loginscreen_gridbox'];
		$this->os_settings['offlinecolor']	= $this->offlinecolor;
		$this->os_settings['onlinecolor']	= $this->onlinecolor;
	}

	public function initOpenSim() {
		$params			= &JComponentHelper::getParams('com_opensim');
		$this->params	= $params;

		$osdbhost		= ($params->get('opensimgrid_dbhost'))   ? $params->get('opensimgrid_dbhost'):$params->get('opensim_dbhost');
		$osdbuser		= ($params->get('opensimgrid_dbuser'))   ? $params->get('opensimgrid_dbuser'):$params->get('opensim_dbuser');
		$osdbpasswd		= ($params->get('opensimgrid_dbpasswd')) ? $params->get('opensimgrid_dbpasswd'):$params->get('opensim_dbpasswd');
		$osdbname		= ($params->get('opensimgrid_dbname'))   ? $params->get('opensimgrid_dbname'):$params->get('opensim_dbname');
		$osdbport		= ($params->get('opensimgrid_dbport'))   ? $params->get('opensimgrid_dbport'):$params->get('opensim_dbport');
		$this->opensim	= new opensim($osdbhost,$osdbuser,$osdbpasswd,$osdbname,$osdbport);
	}

	public function getOSdb() {
		return $this->opensim->_osgrid_db;
	}

	public function getUIDfromUser() {
		$db =& JFactory::getDBO();
		$query = sprintf("SELECT opensimID FROM #__opensim_userrelation WHERE joomlaID = '%d'",$this->currentuser->id);
		$db->setQuery($query);
		$uuid = $db->loadResult();
		if(!$uuid) $this->os_uid = FALSE;
		else $this->os_uid = $uuid;
	}

	public function initFriendsList() {
		if(!$this->_osgrid_db) return $this->externalDBerror();

		$query = $this->opensim->getUserDataQuery($this->os_uid);
		$this->_osgrid_db->setQuery($query['friends']);
		$this->friendlist = $this->_osgrid_db->loadAssocList();
		if(is_array($this->friendlist)) {
			$friends[0] = array();
			$friends[1] = array();
			foreach($this->friendlist AS $key => $friend) {
				$onlinestatus = $this->opensim->getUserPresence($friend['friendid']);
				$friendnamequery = $this->opensim->getUserNameQuery($friend['friendid']);
				$this->_osgrid_db->setQuery($friendnamequery);
				$friendname = $this->_osgrid_db->loadAssoc();
				if(!is_array($friendname) || count($friendname) == 0) {
					// lets see if this is a HG visitor
					$hgtest = explode(";",$friend['friendid']);
					if(count($hgtest) == 4) { // yes, this is a HG user
						$friendname['firstname'] = $hgtest[2];
						$hghost = parse_url($hgtest[1]);
						$friendname['lastname'] = (array_key_exists("host",$hghost)) ? "@".$hghost['host']:"@".$hgtest[1];
					} else {
						// this UUID seems not to exists anymore, lets ignore it
						continue;
					}
				}
				/*if(is_array($friendname)) $this->friendlist[$key] = array_merge($this->friendlist[$key],$friendname);*/
				/*$this->friendlist[$key]['onlinestatus'] = $this->opensim->getUserPresence($friend['friendid']);*/
				/*$this->friendlist[$key]['friendnamequery'] = $friendnamequery;*/
				$count = count($friends[$onlinestatus]);
				$friends[$onlinestatus][$count]['name']	= $this->prepareName($friendname['firstname']." ".$friendname['lastname']);
				$friends[$onlinestatus][$count]['uid']	= $friend['friendid'];
			}
		}
		if(is_array($friends[0])) sort($friends[0]);
		if(is_array($friends[1])) sort($friends[1]);
		$this->friendlist = $friends;
		return $friends;
	}

	public function prepareName($name) {
		if(($this->namelength > 0) && (strlen($name) > $this->namelength)) {
			$name = substr($name,0,$this->namelength)."&#x2026;";
		}
		return $name;
	}

	public function getFriendList() {
		if(count($this->friendlist) > 0) { // return the friends as an array
			return $this->friendlist;
		} else { // no friends found
			return null;
		}
	}

	public function externalDBerror() {
	//	JFactory::getApplication()->enqueueMessage(JText::_(ERROR_NOSIMDB),"error");
		return FALSE;
	}
} //end ModOpenSimFriendsHelper
?>
